/**
 * @Aside.js
 * @author Zhang Xinxu & Zhang Chengrong
 * @Created: 16-10-21
 * @version 0.1.0 add aria  2017-01-17
 */

(function (global, factory) {
    if (typeof exports === 'object' && typeof module !== 'undefined') {
        module.exports = factory();
    } else if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        global.Aside = factory();
    }
}(this, function () {
    'use strict';

    var $ = window.Zepto;

    /**
     * 定制级极简的侧边栏
     * @使用示例
     *  new Aside($('#container > a'), {
     *     onInit: function() {}
     *  });

     * 或者包装器用法
     * $('#container > a').aside({
     *   onInit: function() {}
     * });
     */
    
    /*eslint-disable */

    var ACTIVE = 'active';

    var Aside;

    // 根据属性获得对应的元素
    $.fn.aside = function (options) {
        if (!this.$aside) {
            this.$aside = new Aside(this, options);
        }
    };

    Aside = function (el, options) {
        var defaults = {
            scrollable: '.scrollable',
            onInit: function () {},
            onShow: function () {},
            onHide: function () {},
            onScroll: function () {}
        };

        var elTrigger = $(el);
        var elBody = $(document.body);

        // 如果元素没有，打道回府
        if (elTrigger.length === 0) return;

        var params = $.extend({}, defaults, options);

        // 侧边栏的id
        var idAside = el.data('rel');
        // 侧边栏元素
        var elAside = $('#' + idAside);

        var isActive = function () {
            return location.hash.replace('#&', '') === idAside;
        };

        var self = this;

        self.el = {
            trigger: elTrigger,
            aside: elAside,
            overlay: elAside.find('.aside-overlay'),
            content: elAside.find('.aside-content')
        };

        self.callback = {
            init: params.onInit,
            show: params.onShow,
            hide: params.onHide,
            scroll: params.onScroll
        };

        // for aria
        // 存储标题，在浮层隐藏的时候使用
        self.title = elTrigger.attr('title');
        elTrigger.attr({
            role: 'button',
            'aria-haspopup': 'true'
        });

        // 一些事件
        elTrigger.on('click', function (event) {
            event.preventDefault();
            self.show(this.href);
        });

        // 侧边栏隐藏
        elAside.prependTo(elBody);

        self.el.overlay.on('click', function () {
            self.hide();
        }).on('touchmove', function (event) {
            event.preventDefault();
        }).attr({
            role: 'button'
        });

        // 内滚动
        if (elAside.find(params.scrollable).length) {
            self.scrollable(params.scrollable);
        }

        // 手指右滑出屏幕
        self.swipeOut(params.scrollable);

        // 判断是否默认就需要展开侧边栏
        if (isActive()) {
            self.show();
        }

        window.addEventListener('popstate', function () {
            if (isActive()) {
                self.show();
            } else {
                self.hide(true);
            }
        });
    };

    Aside.prototype.show = function(hash) {
        var self = this;
        // 元素和回调
        var el = self.el, callback = self.callback;

        // 具体点元素
        var elTrigger = el.trigger;
        var elAside = el.aside;

        if (elAside.hasClass(ACTIVE)) {
            return;
        }

        // 侧边栏进入
        elAside.addClass(ACTIVE);

        // aria提示
        el.overlay.attr('title', '点击关闭浮层');
        elTrigger.attr('title', '浮层已显示');

        // 拉取数据
        if (!elTrigger.data('init')) {
            elTrigger.data('init', true);
            callback.init.call(self);
        }

        $('html').addClass('noscroll');

        // elTrigger.css('display', 'none');
        callback.show.call(self);

        // 记录，以便刷新的时候依然打开最近阅读
        if (history.pushState && hash) {
            history.pushState(null, null, hash);
        }
    };

    Aside.prototype.hide = function(isFromPopState) {
        var self = this;
        // 元素和回调
        var el = self.el, callback = self.callback;

        // 具体的元素
        var elTrigger = el.trigger;
        var elAside = el.aside;

        $('html').removeClass('noscroll');

        // 需要当前是显示的
        if (!elAside.hasClass(ACTIVE)) {
            return;
        }

        elAside.removeClass(ACTIVE);

        // aria提示
        el.overlay.attr('title', '浮层已关闭');
        elTrigger.attr('title', self.title);

        // elTrigger.css('display', 'block');
        callback.hide.call(self);

        // 地址还原
        if (history.pushState && !isFromPopState) {
            var href = location.href.split('#')[0];

            history.go(-1);

            setTimeout(function () {
                history.replaceState(null, null, href);
            }, 0);
        }
    };

    Aside.prototype.scrollable = function(selectorScrollable) {
        var self = this;

        var elAside = self.el.aside;

        // 如果没有滚动容器选择器，或者已经绑定了滚动时间，忽略
        if (!selectorScrollable || elAside.data('isBindScroll')) {
            return self;
        }

        // 是否是搓浏览器
        var isSBBrowser = /mx\d.*mqqbrowser/i.test(navigator.userAgent);

        var data = {
            posY: 0,
            maxscroll: 0
        };

        // 手动触发滚动
        elAside.on('touchstart', selectorScrollable, function (event) {
            var events = event.touches[0] || event;

            // 当前滚动元素标记
            data.elScroll = $(this);

            // 垂直位置标记
            data.posY = events.pageY;
            data.scrollY = data.elScroll.scrollTop();
            // 是否可以滚动
            data.maxscroll = this.scrollHeight - this.clientHeight;
        });
        elAside.on('touchmove', function (event) {
            // 如果不足于滚动，则禁止触发整个窗体元素的滚动
            if (data.maxscroll <= 0 || isSBBrowser) {
                // 禁止滚动
                event.preventDefault();
            }
            // 滚动元素
            var elScroll = data.elScroll;
            // 当前的滚动高度
            var scrollTop = elScroll.scrollTop();

            // 现在移动的垂直位置，用来判断是往上移动还是往下
            var events = event.touches[0] || event;
            // 移动距离
            var distanceY = events.pageY - data.posY;

            if (isSBBrowser) {
                elScroll.scrollTop(data.scrollY - distanceY);
                elScroll.trigger('scroll');
                return;
            }

            // 上下边缘检测
            if (distanceY > 0 && scrollTop == 0) {
                // 往上滑，并且到头
                // 禁止滚动的默认行为
                event.preventDefault();
                return;
            }

            // 下边缘检测
            if (distanceY < 0 && (scrollTop + 1 >= data.maxscroll)) {
                // 往下滑，并且到头
                // 禁止滚动的默认行为
                event.preventDefault();
                return;
            }

            self.callback.scroll.call(self, elScroll);
        });

        // 防止多次重复绑定
        elAside.data('isBindScroll', true);
    };

    Aside.prototype.swipeOut = function(selectorScrollable) {
        var self = this;

        var elAside = self.el.aside;
        // 滑动容器
        var elContent = self.el.content;

        var data = {};

        elContent.on('touchstart', function (event) {
            var events = event.touches[0] || event;
            // 垂直位置标记
            data.posX = events.pageX;
            data.distanceX = 0;
            // 动画中不执行
            if (data.transition) {
                return;
            }

            // 干掉transition动画
            elContent.css({
                '-webkit-transition': 'none',
                transition: 'none'
            });

            var target = event.target, elTarget = $(target);
            var elScroll = elTarget.parents(selectorScrollable);

            if (elTarget.is(selectorScrollable)) {
                elScroll = elTarget;
            }
            if (elScroll.length && elScroll[0].scrollHeight > elScroll[0].clientHeight) {
                // 如果是滚动容器，且可以滚动
                data.moving = false;
                return;
            }

            data.moving = true;
        });

        elAside.on('touchmove', function (event) {
            var events = event.touches[0] || event;

            var distanceX = events.pageX - data.posX;
            data.distanceX = distanceX;

            if (distanceX < 0) {
                distanceX = 0;
            }

            if (!data.moving) {
                // 检测是否可以跟随移动
                return;
            }

            // 跟随移动
            elContent.css({
                '-webkit-transform': 'translateX('+ distanceX +'px)',
                transform: 'translateX('+ distanceX +'px)',
            });

            event.preventDefault();
        });

        elAside.on('touchend', function () {
            // 开启transition动画
            elContent.css({
                '-webkit-transition': '',
                transition: ''
            });

            if (!data.moving) {
                return;
            }

            data.transition = true;

            // 水平移动超过100像素，移出
            if (data.distanceX > 100) {
                self.hide();
                // 移出
                elContent.css({
                    '-webkit-transform': 'translateX(100%)',
                    transform: 'translateX(100%)',
                });
            } else {
                // 位置还原
                elContent.css({
                    '-webkit-transform': 'translateX(0)',
                    transform: 'translateX(0)',
                });
            }

            // 动画结束还原可touchstart
            setTimeout(function () {
                // 一切如初
                elContent.css({
                    '-webkit-transform': '',
                    transform: '',
                });
                delete data.transition;
            }, 151);

            data.moving = false;
        });

        return self;
    };

    return Aside;
}));
