/**
 * @Popup.js
 * @author zhangxinxu
 * @version
 * @Created: 16-11-14
 * @description 自下而上的居底浮层操作
 * @document  /guide
 */
(function (global, factory) {
    if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        global.Popup = factory();
    }
}(this, function () {
    'use strict';

    var ACTIVE = 'active';

    var Popup = function (content, options) {
        var defaults = {
            overlay: true,
            onShow: function () {},
            onHide: function () {}
        };

        var params = $.extend({}, defaults, options || {});

        var self = this;

        self.callback = {
            show: params.onShow,
            hide: params.onHide
        };

        // 创建内容
        var aside = $('<aside class="aside"></aside>');
        // 黑色蒙层
        var overlay = $('<i class="aside-overlay"></i>').on('touchmove', function (event) {
            event.preventDefault();
        });

        // aria
        overlay.attr({
            role: 'button'
        });

        // 内容区域
        var popup = $('<div class="aside-popup"></div>');

        if (typeof content == 'string') {
            popup.html(content);
        } else if (content && content.size) {
            popup.append(content);
        } else {
            return self;
        }

        aside.append(overlay).append(popup);

        // 元素事件
        overlay.on('click', function () {
            self.hide();
        });

        // 暴露
        self.el = {
            body: $('body'),
            aside: aside,
            overlay: overlay,
            popup: popup
        };

        // 载入页面
        self.el.body.append(aside);

        // 内部滚动
        self.scrollable('.scrollable');

        // 元素合体
        if (params.overlay === false) {
            aside.css({
                top: 'auto',
                height: popup.height()
            });
            overlay.css('display', 'none');
        }

        self.overlay = params.overlay;

        // 显示
        aside[0].offsetWidth;

        // 显示
        self.show();
    };

    // Popup的内滚动处理
    Popup.prototype.scrollable = function(selectorScrollable) {
        var self = this;

        var elAside = self.el.aside;

        // 如果没有滚动容器选择器，或者已经绑定了滚动时间，忽略
        if (!selectorScrollable || elAside.data('isBindScroll')) {
            return self;
        }

        var data = {
            posY: 0,
            maxscroll: 0
        };

        var isSBBrowser = /mx\d.*mqqbrowser/i.test(navigator.userAgent);

        // 手动触发滚动
        elAside.on('touchstart', selectorScrollable, function (event) {
            var events = event.touches[0] || event;

            // 当前滚动元素标记
            data.elScroll = $(this);

            // 垂直位置标记
            data.posY = events.pageY;
            data.scrollY = data.elScroll.scrollTop();
            // 是否可以滚动
            data.maxscroll = this.scrollHeight - this.clientHeight;

        });
        elAside.on('touchmove', function (event) {
            // 如果不足于滚动，则禁止触发整个窗体元素的滚动
            if (data.maxscroll <= 0 || isSBBrowser) {
                // 禁止滚动
                event.preventDefault();
            }
            // 滚动元素
            var elScroll = data.elScroll;
            // 当前的滚动高度
            var scrollTop = elScroll.scrollTop();

            // 现在移动的垂直位置，用来判断是往上移动还是往下
            var events = event.touches[0] || event;
            // 移动距离
            var distanceY = events.pageY - data.posY;

            if (isSBBrowser) {
                elScroll.scrollTop(data.scrollY - distanceY);
                elScroll.trigger('scroll');
                return;
            }

            // 上下边缘检测
            if (distanceY > 0 && scrollTop == 0) {
                // 往上滑，并且到头
                // 禁止滚动的默认行为
                event.preventDefault();
                return;
            }

            // 下边缘检测
            if (distanceY < 0 && (scrollTop + 1 >= data.maxscroll)) {
                // 往下滑，并且到头
                // 禁止滚动的默认行为
                event.preventDefault();
                return;
            }
        });

        // 防止多次重复绑定
        elAside.data('isBindScroll', true);
    };

    Popup.prototype.show = function () {
        var self = this;

        // 元素
        var el = self.el;

        // 回调
        var callback = self.callback;

        if (el.aside) {
            el.aside.addClass(ACTIVE);

            if (el.overlay) {
                el.overlay.attr('title', '点击关闭浮层');
            }

            if (callback && typeof callback.show == 'function') {
                callback.show.call(self, el);
            }

            if (self.overlay) {
                $('html').addClass('noscroll');
            }
        }

        return self;
    };

    Popup.prototype.hide = function () {
        var self = this;

        // 元素
        var el = self.el;

        // 回调
        var callback = self.callback;

        if (el.aside) {
            el.aside.removeClass(ACTIVE);

            if (el.overlay) {
                el.overlay.attr('title', '浮层已关闭');
            }

            if (callback && typeof callback.hide == 'function') {
                callback.hide.call(self, el);
            }

            $('html').removeClass('noscroll');
        }

        return self;
    };

    Popup.prototype.remove = function () {
        var self = this;

        // 元素
        var el = this.el;

        if (el.aside) {
            el.aside.remove();
        }
    };

    return Popup;
}));
