/**
 * 过滤器逻辑，提供分类等过滤数据的初始化、固定展示栏展示等功能
 * 可以通过实现 onFilterChange 来处理过滤条件更新逻辑
 *
 * @author Zhang Chengrong
 *
 * @html
 * <% if (data.filters && data.filters.forEach) { %>
 * <!-- 筛选条件 S -->
 * <!-- 标签 -->
 * <div id="filters" class="toplist-tag-x">
 *     <% data.filters.forEach(function (filter) { %>
 *     <div class="toplist-tag jsFilter" data-key="<%= filter.key %>">
 *         <% filter.items.forEach(function (d, i) { %><a href="javascript:"
 *             class="btn-tag<%= i == filter.selectedIndex ? ' active' : '' %> jsTag"
 *             data-value="<%= d.value %>"><%= d.text %></a><% }) %>
 *     </div>
 *     <% }) %>
 * </div>
 * <!-- 滚动后出现的固定栏 -->
 * <div id="filtersBar" class="toplist-fixed-bar">
 *     <output id="filtersBarTexts" class="red"></output>
 *     <svg class="icon icon-arrow-r"><use xlink:href="#icon-arrow-r"></use></svg>
 * </div>
 * <!-- 筛选条件 E -->
 * <% } %>
 */
define(function (require, exports, module) {
    var $ = require('lib.Zepto');

    exports = module.exports = {
        // 过滤器初始化
        init: function () {
            if ($('#filters').length) {
                // 过滤条件
                this.initFilters();

                if ($('#filtersBar').length) {
                    // 滚动标题栏交互
                    this.initFiltersBar();
                    // 点击显示过滤条件们
                    this.initToggleFilter();
                }
            }
        },

        // 查询条件初始化回调，需要被重写
        onFilterInit: function (filterKey, filterValue) {
            console.warn('查询条件初始化:', filterKey + '=' + filterValue); // eslint-disable-line no-console
        },

        // 查询条件更新回调，需要被重写
        onFilterChange: function (filterKey, filterValue) {
            console.warn('查询条件更新:', filterKey + '=' + filterValue); // eslint-disable-line no-console
        },

        // 查询条件取消回调，需要被重写
        onFilterCancel: function (filterKey) {
            console.warn('查询条件取消:', filterKey); // eslint-disable-line no-console
        },

        onFiltersReset: function (filterKeys) {
            console.warn('查询条件重置:', filterKeys); // eslint-disable-line no-console
        },

        // 过滤器
        initFilters: function () {
            var self = this;

            var sortFilterKeys = [];
            var filters = {};

            // 一些元素
            // 固定栏
            var elFixedBar = $('#filtersBar');
            var elBarTexts = $('#filtersBarTexts');
            // 筛选栏
            var elFilters = $('#filters');

            // 更新滚动标题文本
            function updateSelectedTexts() {
                if (elBarTexts.length) {
                    var texts = [];

                    sortFilterKeys.forEach(function (filterKey) {
                        if (filters[filterKey] && filters[filterKey].value !== undefined) {
                            texts.push(filters[filterKey].text);
                        }
                    });

                    elBarTexts.text(texts.join('－'));
                }
            }

            elFilters.find('.jsFilter').each(function () {
                var elFilter = $(this);
                var elTags = elFilter.find('.jsTag');
                var elActiveTag = elTags.filter('.active');

                var filterKey = elFilter.data('key');

                var filter = filters[filterKey] = {
                    value: elActiveTag.data('value'),
                    text: elActiveTag.text(),
                };

                sortFilterKeys.push(filterKey);

                if (elActiveTag.length) {
                    self.onFilterInit(filterKey, filter.value);
                }

                // aria作为选项卡处理
                elTags.attr('role', 'tab');
                elActiveTag.attr('aria-selected', 'true');
            });

            elFilters.on('click', '.jsTag', function () {
                var elTag = $(this);
                var elFilter = elTag.parents('.jsFilter');

                var filterKey = elFilter.data('key');
                var cancelable = elFilter.attr('cancelable') != null;

                var filter = filters[filterKey];

                // 下拉浮层筛选收起
                elFilters.css({
                    '-webkit-transition': 'none',
                    transition: 'none'
                });
                elFixedBar.trigger('click');
                clearTimeout(self.timerFilterTop);
                elFilters.css({
                    top: 0,
                    opacity: 1
                });
                setTimeout(function () {
                    elFilters.removeAttr('style');
                }, 0);
                if (elTag.hasClass('active')) {
                    if (cancelable) {
                        elTag.removeClass('active').attr('aria-selected', 'false');

                        delete filter.value;
                        delete filter.text;

                        self.onFilterCancel(filterKey);
                    }
                    return;
                }

                elTag.parent().children().removeClass('active').attr('aria-selected', 'false');
                elTag.addClass('active').attr('aria-selected', 'true');

                filter.value = elTag.data('value');
                filter.text = elTag.text();
                self.onFilterChange(filterKey, filter.value);

                updateSelectedTexts();
            });


            $('#resetAsideFilter').on('click', function () {
                var filterKeys = $('.jsFilter').map(function () {
                    return $(this).data('key');
                });
                self.onFiltersReset(filterKeys);
            });

            $('#closeAsideFilter').on('click', function () {
                self.ASIDE.hide();
            });

            updateSelectedTexts();
        },

        // 标题栏滚动交互
        initFiltersBar: function () {
            // 标签栏
            var elTagX = $('#filters');
            // 固定栏
            var elFixedBar = $('#filtersBar');

            // 滚动容器
            var elContainer = $(g_data.container || window);

            // 距离顶部的大小
            var offsetTopTagX = elTagX.offset().top;
            // 本身的高度
            var heightTagX = elTagX.height();

            // 根据滚动位置确定交互
            var fnUIBar = function () {
                var scrollTop = elContainer.scrollTop();
                // 如果滚动高度小于距离高度，一切还原
                var minScrollTop = offsetTopTagX;
                var maxScrollTop = heightTagX;

                var opacity = (maxScrollTop - scrollTop) / (heightTagX - offsetTopTagX);

                if (scrollTop < minScrollTop) {
                    // 100%固定条隐藏
                    elFixedBar.css({
                        opacity: 0,
                        visibility: 'hidden'
                    });
                    elTagX.css({
                        opacity: 1
                    });
                } else if (scrollTop > maxScrollTop) {
                    // 100%呈现
                    elFixedBar.css({
                        opacity: 1,
                        visibility: 'visible'
                    });
                    elTagX.css({
                        opacity: 0
                    });
                } else {
                    // 滚动区间，根据数值动态计算固定栏的透明度
                    elFixedBar.css({
                        opacity: 1 - opacity,
                        visibility: 'visible'
                    });
                    // 筛选标签容器
                    elTagX.css({
                        opacity: opacity
                    });
                }
            };

            if (elTagX.length && elFixedBar.length) {
                fnUIBar();

                elContainer.on('scroll', fnUIBar);

                // 初始化里面的数据
                var tagSelected = $.map(elTagX.find('.active'), function (ele) {
                    return $(ele).html();
                }).join('－');
                elFixedBar.find('output').html(tagSelected);
            }

            window.addEventListener('resize', function () {
                // 高度重新获取
                heightTagX = elTagX.height();
            });
        },
        // 固定栏点击显示筛选条件
        initToggleFilter: function () {
            var self = this;
            // 固定栏
            var elFixedBar = $('#filtersBar');
            // 筛选条件
            var elFilters = $('#filters');
            // 滚动容器
            var elContainer = $(g_data.container || window);

            // 类名
            var ACTIVE = 'active';

            // 可否滚动，针对部分比较糟糕的Android
            var isScrollAble = true;

            if (!elFixedBar.length || !elFilters.length) {
                return this;
            }

            elFixedBar.on('click', function () {
                var opacity = +elFixedBar.css('opacity');

                if (opacity === 1) {
                    // 滚动高度
                    var scrollTop = elContainer.scrollTop();
                    // 列表高度
                    var heightFilters = elFilters.height();
                    // 条子高度
                    // var heightBar = elFixedBar.height();

                    // 定位
                    if (elFixedBar.hasClass(ACTIVE)) {
                        // 三角变化
                        elFixedBar.removeClass(ACTIVE);
                        // 隐藏
                        elFilters.css({
                            '-webkit-transform': 'translate(0, 0%)',
                            transform: 'translateY(0%)'
                        });
                        self.timerFilterTop = setTimeout(function () {
                            elFilters.css({
                                top: 0,
                                opacity: 0
                            });
                        }, 150);

                        isScrollAble = true;
                    } else {
                        elFixedBar.addClass(ACTIVE);
                        // 过滤定位
                        elFilters.css({
                            opacity: 1,
                            top: scrollTop - heightFilters
                        });
                        setTimeout(function () {
                            elFilters.css({
                                '-webkit-transform': 'translate(0, 100%)',
                                transform: 'translateY(100%)'
                            });
                        }, 0);

                        isScrollAble = false;
                    }
                }
            });
            // 点击页面空白位置，收起展开列表
            $(document).on('touchstart', function (event) {
                var target = event.target;
                if (target && elFixedBar.hasClass(ACTIVE) &&
                    target.id !== elFixedBar.attr('id') && target.id !== elFilters.attr('id') &&
                    elFilters[0].contains(target) === false && elFixedBar[0].contains(target) === false) { // eslint-disable-line max-len
                    event.preventDefault();
                    elFixedBar.trigger('click');
                }
            });
            $(document).on('touchmove', function (event) {
                if (isScrollAble === false) {
                    event.preventDefault();
                }
            });
        },
    };
});
