/*!
 * qdsdk.js v1.0.0
 * (c) 2016 Zhang Chengrong
 * Released under the MIT License.
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.qdsdk = factory());
}(this, function () { 'use strict';

  var log = void 0; // eslint-disable-line import/no-mutable-exports

  if ('development' !== 'production') {
    (function () {
      var hasConsole = typeof console !== 'undefined';

      log = function log(msg) {
        if (hasConsole) {
          console.log('[QDSDK log]: ' + msg); // eslint-disable-line no-console
        }
      };
    })();
  }

  /* eslint-disable max-len */

  // Mozilla/5.0 (Linux; Android 4.4.4; en-us; Nexus 5 Build/JOP40D) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2307.2 Mobile Safari/537.36
  var REG_ANDROID = /\bAndroid ([\d\.]+)/;

  // iPhone: Mozilla/5.0 (iPhone; CPU iPhone OS 8_0 like Mac OS X) AppleWebKit/600.1.3 (KHTML, like Gecko) Version/8.0 Mobile/12A4345d Safari/600.1.4
  // iPad:   Mozilla/5.0 (iPad; CPU OS 7_0 like Mac OS X) AppleWebKit/537.51.1 (KHTML, like Gecko) Version/7.0 Mobile/11A465 Safari/9537.53
  var REG_IOS = /\b(iPad|iPhone|iPod)\b.*? OS ([\d_]+)/;

  // Mozilla/5.0 (iPad; CPU OS 8_0 like Mac OS X) AppleWebKit/600.1.4 (KHTML, like Gecko) Mobile/12A365/QDReaderiPad/3.7.0/160
  var REG_IPAD_CLIENT = /\/QDReaderiPad\/([\d\.]+)/;

  // Mozilla/5.0 (iPhone; CPU iPhone OS 9_3 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Mobile/13E230/QDReaderiOS/3.7.0/160/QDReaderAppstore
  var REG_IPHONE_CLIENT = /\/QDReaderiOS\/([\d\.]+)/;

  // Mozilla/5.0 (Linux; U; Android 4.4.4; zh-cn; MX4 Pro Build/KTU84P) AppleWebKit/533.1 (KHTML, like Gecko)Version/4.0 MQQBrowser/5.4 TBS/025489 Mobile Safari/533.1 QDReaderAndroid/6.3.3/244/cepingtuan/865863029948945
  var REG_ANDROID_CLIENT = /\bQDReaderAndroid\/([\d\.]+)/;

  /* eslint-enable max-len */

  var SCHEMA = 'jsbridge'; // SDK协议头

  var ua = navigator.userAgent;

  var noop = function noop(d) {
    return d;
  };

  var isFunction = function isFunction(obj) {
    return typeof obj === 'function';
  };
  var isArray = Array.isArray;

  var apiSupports = {}; // 缓存所有 api support
  var apiFunctions = {}; // 缓存所有 api function
  var apiEvents = {}; // 缓存所有 api eventName

  var callbackUUID = 0; // 回调方法唯一序号
  var generatorCallbackId = function generatorCallbackId() {
    return ++callbackUUID;
  }; // 回调方法ID生成器
  var allCallbacks = {}; // 缓存所有回调方法

  /*
   * 生成 bridge url
   * @param  {String} module - 模块
   * @param  {String} method - 方法
   * @param  {Object} params - 参数
   * @param  {Number} callbackId - 回调方法ID
   * @return {String} bridge url
   */
  function generatorBridgeUrl(module, method, params, callbackId) {
    var encode = encodeURIComponent;

    var host = encode(module);
    var port = callbackId ? ':' + callbackId : '';
    var path = encode(method);
    var queryString = params ? '?query=' + encode(JSON.stringify(params)) : '';

    return SCHEMA + '://' + host + port + '/' + path + queryString;
  }

  /**
   * isAndroid
   * @description 如果在 android 中，值为 true，否则为 false
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var isAndroid = REG_ANDROID.test(ua);

  /**
   * @attribute isIOS
   * @description 如果在 iOS 中，值为 true，否则为 false
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var isIOS = !isAndroid && REG_IOS.test(ua); // 防止某些安卓机型的 ua 带 iPhone

  // 获取客户端平台信息和版本信息
  var clientInfo = function () {
    var plat = 'browser';
    var ver = '0';

    if (isAndroid) {
      var androidMatch = ua.match(REG_ANDROID_CLIENT);

      if (androidMatch) {
        plat = 'android';
        ver = androidMatch[1] || ver;
      }
    } else if (isIOS) {
      var iPadMatch = ua.match(REG_IPAD_CLIENT);
      var iPhoneMatch = ua.match(REG_IPHONE_CLIENT);

      if (iPadMatch) {
        plat = 'iPad';
        ver = iPadMatch[1] || ver;
      } else if (iPhoneMatch) {
        plat = 'iPhone';
        ver = iPhoneMatch[1] || ver;
      } else {
        plat = 'iPhone';
      }
    }

    return { platform: plat, version: ver };
  }();

  /**
   * @attribute platform
   * @description 当前运行平台: browser|android|iPad|iPhone
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var platform = clientInfo.platform;

  /**
   * @attribute clientVersion
   * @description 客户端版本
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var clientVersion = clientInfo.version;

  /**
   * 比较版本号
   * @param  {String} ver1 - 版本号
   * @param  {String} ver2 - 需要比较的版本号，默认为当前客户端版本号
   * @return {Number} 当 ver1 < ver2 返回-1, 当 ver1 == ver2 返回0, 当 ver1 > ver2 返回1
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * compare('1.0.1'); // 返回-1表示小于当前客户端版本，0表示等于当前客户端版本，1表示大于当前客户端版本
   * compare('1.0.1', '1.0.2'); // 返回-1
   */
  function compare(ver1, ver2) {
    if (!ver2) {
      ver2 = clientVersion;
    }

    var arr1 = String(ver1).split('.');
    var arr2 = String(ver2).split('.');

    var len = Math.max(arr1.length, arr2.length);
    var i = void 0;

    var l = void 0;
    var r = void 0;

    for (i = 0; i < len; i++) {
      l = isFinite(arr1[i]) && Number(arr1[i]) || 0;
      r = isFinite(arr2[i]) && Number(arr2[i]) || 0;

      if (l < r) {
        return -1;
      } else if (l > r) {
        return 1;
      }
    }

    return 0;
  }

  /**
   * 检查当前客户端版本是否支持某接口
   * @param  {String} name - 接口名称，格式：module.method
   * @return {Boolean} 返回 true 则表示支持该接口，false 则表示不支持该接口
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * support('device.getSystemVersion');
   */
  function support(name) {
    var supportVersion = apiSupports[name];

    if (!supportVersion) {
      return false;
    }

    var vers = supportVersion.split('-');

    if (vers.length === 2) {
      return compare(vers[0]) < 1 && compare(vers[1]) > -1;
    }

    return compare(vers[0]) < 1;
  }

  /**
   * 通过请求 url 来与客户端交互
   * @param {String} url - 请求地址
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * invokeUrl('schema://do/some/action');
   */
  function invokeUrl(url) {
    var body = document.body || document.documentElement;

    var iframe = document.createElement('iframe');
    iframe.style.cssText = 'display:none;width:0px;height:0px;';

    function failCallback() {
      /*
        正常情况下是不会回调到这里的, 只有客户端没有捕获这个 url 请求,
        浏览器才会发起 iframe 的加载, 但这个 url 实际上是不存在的,
      */
      console.error('Client not catch!'); // eslint-disable-line no-console
    }

    if ('development' !== 'production') {
      log('Invoke Url -> ' + decodeURIComponent(url));
    }

    /*
      ios 必须先赋值, 然后 append, 否者连续的 api 调用会间隔着失败
      也就是 api1(); api2(); api3(); api4(); 的连续调用,
      只有 api1 和 api3 会真正调用到客户端
    */
    if (isIOS) {
      iframe.onload = failCallback;
      iframe.src = url;
    }

    body.appendChild(iframe);

    /*
      android 必须先添加到页面, 然后再绑定 onload 和设置 src
      1. 先设置 src 再 append 到页面, 会导致在接口回调(callback)中嵌套调用 api会失败,
          iframe会直接当成普通url来解析
      2. 先设置onload 在 append , 会导致 iframe 先触发一次 about:blank 的 onload 事件
     */
    if (isAndroid) {
      iframe.onload = failCallback;
      iframe.src = url;
    }

    // 删除 iframe
    setTimeout(function () {
      return body.removeChild(iframe);
    }, 0);
  }

  /**
   * 调用客户端
   * @param {String} module - 模块
   * @param {String} method - 方法
   * @param {Object} params - 参数
   * @param {Function} callback - 回调
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * invokeClient('device', 'systemVersion', params, callback);
   */
  function invokeClient(module, method, params, callback) {
    if (!module || !method) {
      return;
    }

    //用于本地（端外）跳转
    // if(~location.protocol.indexOf("http") && ~location.href.indexOf("local") && params && params.url){
    //   location.href = params.url;
    //   return;
    // }
    
    var callbackId = void 0;

    if (isFunction(params)) {
      callback = params;
      params = null;
    }

    if (isFunction(callback)) {
      callbackId = generatorCallbackId();
      allCallbacks[callbackId] = function () {
        var _arguments = arguments;

        setTimeout(function () {
          return callback.apply(null, _arguments);
        }, 0); // 异步执行回调
      };
    }

    var url = generatorBridgeUrl(module, method, params, callbackId);

    invokeUrl(url);
  }

  /*
   * 调用回调方法
   * @param {Number} callbackId - 回调方法ID
   * @param {Object} result - 回调结果
   * @param {Number} result.result 回调状态(0表示正常)
   * @param {Object} result.data 回调数据
   */
  function execCallback(callbackId, result) {
    var callback = allCallbacks[callbackId];

    if ('development' !== 'production') {
      log('Exec Callback -> ' + callbackId);
    }

    if (isFunction(callback)) {
      callback(result);
    }
  }

  var generatorEventCallbackName = function generatorEventCallbackName(eventName) {
    return 'evt-' + eventName;
  };

  /*
   * 注册事件
   * @param {String} name 事件名称
   * @param {Object} options 配置项
   * @param {Object} options.support 支持配置
   */
  function registerEvent(name, options) {
    if (apiEvents[name]) {
      return;
    }

    var plat = platform;
    var supp = options.support;

    if (!supp[plat]) {
      // 没有指定特殊平台，则尝试用通用的 iOS 和android 去找
      if (isIOS && supp.iOS) {
        plat = 'iOS';
      } else if (isAndroid && supp.android) {
        plat = 'android';
      }
    }

    if (supp[plat]) {
      invokeClient('event', 'add', { name: name });

      var moduleMethod = 'event.' + name;

      // 用于 supportVersion 判断
      apiSupports[moduleMethod] = supp[plat];

      // 缓存事件名
      apiEvents[name] = true;
    }
  }

  /**
   * 添加事件监听
   * @param {String} eventName - 事件名称
   * @param {Function} handler - 处理方法
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  function addEventListener(eventName, handler) {
    var callbackName = generatorEventCallbackName(eventName);

    if ('development' !== 'production') {
      log('Add Event Listener -> ' + eventName);
    }

    if (!allCallbacks[callbackName]) {
      allCallbacks[callbackName] = [];
    }

    allCallbacks[callbackName].push(handler);
  }

  /**
   * 移除事件监听
   * @param {String} eventName - 事件名称
   * @param {Function} handler - 处理方法
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  function removeEventListener(eventName, handler) {
    var callbackName = generatorEventCallbackName(eventName);
    var handlers = allCallbacks[callbackName];

    if ('development' !== 'production') {
      log('Remove Event Listener -> ' + eventName);
    }

    if (!handlers) {
      return;
    }

    if (!handler) {
      delete allCallbacks[callbackName];
      return;
    }

    var i = void 0;

    for (i = handlers.length - 1; i >= 0; i--) {
      if (handler === handlers[i]) {
        handlers.splice(i, 1);
      }
    }
  }

  /*
   * 执行事件监听
   * @param {String} eventName - 事件名称
   * @param {Object} result - 事件结果
   * @param {Number} result.result 事件状态(0表示正常)
   * @param {Object} result.data 事件数据
   */
  function dispatchEvent(eventName, result) {
    var callbackName = generatorEventCallbackName(eventName);
    var handlers = allCallbacks[callbackName];

    if ('development' !== 'production') {
      log('Dispatch Event -> ' + eventName);
    }

    if (isArray(handlers)) {
      handlers.forEach(function (handler) {
        return handler(result);
      });
    }
  }

  // SDK 根节点
  var sdkRoot = {
    __supports: apiSupports,
    __functions: apiFunctions,
    __events: apiEvents,
    __callbacks: allCallbacks,

    isAndroid: isAndroid,
    isIOS: isIOS,
    platform: platform,
    clientVersion: clientVersion,

    compare: compare,
    support: support,

    invokeUrl: invokeUrl,
    invokeClient: invokeClient,

    addEventListener: addEventListener,
    removeEventListener: removeEventListener
  };

  /**
   * 挂载 api 方法, 把指定 api 包装为固定的调用形式
   * @param {String} module - 模块名
   * @param {String} method - 方法名
   * @param {String} options - 方法配置
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * jssdk.mount('device', 'getSystemVersion', {
   *   android: function () {},
   *   iOS: function () {},
   *
   *   AndroidQidian: function () {},
   *   iPhoneQidian: function () {},
   *   iPadQidian: function () {},
   *   browser: function () {}, // 某些 api 可能有浏览器兼容的方式
   *   support: {
   *     AndroidQidian: '1.0.0',
   *     iPhoneQidian: '1.0.0',
   *     iPadQidian: '1.0.0',
   *   },
   * });
   */
  function mount(module, method, options) {
    if (!sdkRoot[module]) {
      sdkRoot[module] = {};
    }

    if (sdkRoot[module][method]) {
      return;
    }

    var plat = platform;
    var func = options[plat];

    if (!func) {
      // 没有指定特殊平台，则尝试用通用的 iOS 和android 去找
      if (isIOS && options.iOS) {
        plat = 'iOS';
        func = options.iOS;
      } else if (isAndroid && options.android) {
        plat = 'android';
        func = options.android;
      } else {
        func = noop;
      }
    }

    sdkRoot[module][method] = func;

    var moduleMethod = module + '.' + method;

    // 用于 supportVersion 判断
    if (options.support[plat]) {
      apiSupports[moduleMethod] = options.support[plat];
    }

    // 缓存方法
    apiFunctions[moduleMethod] = func;
  }

  sdkRoot.mount = mount;

  /**
   * @event browserTitleBarClick
   * @description 点击标题栏事件，监听后点击手机标题栏就会收到通知(可以用来实现点击标题滚动到顶部的功能)
   * @param {Function} callback 事件回调
   * @param {Number} callback.result 事件状态(0表示正常)
   * @param {Object} callback.data 事件数据
   *
   * @example
   * qdsdk.addEventListener("browserTitleBarClick", function(data) {
   *   console.log("Receive event: browserTitleBarClick, data: " + JSON.stringify(data));
   * });
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var browserTitleBarClick = {
    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @event browserVisibilityChange
   * @description 当前页面可见性更改事件，监听后离开/回到本页面就会收到通知(可以用来实现登录成功之后刷新页面的功能)
   * @param {Function} callback 事件回调
   * @param {Number} callback.result 事件状态(0表示正常)
   * @param {Object} callback.data 事件数据
   * @param {Boolean} callback.data.visibility 页面可见性(true: 可见; false: 不可见)
   *
   * @example
   * qdsdk.addEventListener("browserVisibilityChange", function(data) {
   *   console.log("Receive event: browserVisibilityChange, data: " + JSON.stringify(data));
   * });
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var browserVisibilityChange = {
    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  registerEvent('browserTitleBarClick', browserTitleBarClick);

  registerEvent('browserVisibilityChange', browserVisibilityChange);

  /**
   * @method addToBookshelf
   * @description 加入书架
   * @param {Object} params 参数对象
   * @param {Number} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {String} params.author 作者名称
   * @param {Function} callback 回调函数(iOS端3.8.0才支持回调)
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Boolean} callback.data.addToBookshelf 加入书架结果(true: 加入成功; false: 加入失败)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * addToBookshelf({ bookId: 1115277, bookName: '斗罗大陆', author: '唐家三少' });
   */
  var addToBookshelf = {
    iOS: function iOS(params, callback) {
      invokeClient('app', 'addToBookshelf', params, callback);
    },
    android: function android(params, callback) {
      invokeClient('app', 'addToBookshelf', params, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method buyChapter
   * @description 打开某书章节购买页
   * @param {Number} bookId 书籍ID
   * @param {Number} chapterId 章节ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * buyChapter(1115277, 22525239);
   */
  var buyChapter = {
    iOS: function iOS(bookId, chapterId) {
      invokeClient('app', 'buyChapter', { bookId: bookId, chapterId: chapterId });
    },
    android: function android(bookId, chapterId) {
      invokeClient('app', 'buyChapter', { bookId: bookId, chapterId: chapterId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method buyVolume
   * @description 打开某书分册购买页
   * @param {Number} bookId 书籍ID
   *
   * @support android 6.4.0
   *
   * @example
   * buyVolume(1115277);
   */
  var buyVolume = {
    android: function android(bookId) {
      invokeClient('app', 'buyVolume', { bookId: bookId });
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method getLatestBook
   * @description 获取当天最新阅读的一本书
   * @param {Boolean} isInBookshelf 是否只获取书架中的书(iOS端3.8.0此参数无效，只会取书架中的书)
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.bookId 书籍ID
   * @param {String} callback.data.bookName 书籍名称
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getLatestBook = {
    iOS: function iOS(isInBookshelf, callback) {
      invokeClient('app', 'latestBook', { isInBookshelf: isInBookshelf }, callback);
    },
    android: function android(isInBookshelf, callback) {
      invokeClient('app', 'latestBook', { isInBookshelf: isInBookshelf }, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method getReadingDuration
   * @description 获取当天阅读时长
   * @param {Boolean} isTotal 是否获取所有书的阅读时长(false: 只获取书架中书的阅读时长; true: 获取所有书的阅读时长)
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.readingDuration 阅读时长(单位为秒)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getReadingDuration = {
    iOS: function iOS(isTotal, callback) {
      invokeClient('app', 'readingDuration', { isTotal: isTotal }, callback);
    },
    android: function android(isTotal, callback) {
      invokeClient('app', 'readingDuration', { isTotal: isTotal }, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method getReadingPrefer
   * @description 获取用户当前设定男女频道
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.readingPrefer 男女频道(1: 男生频道; 2: 女生频道)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getReadingPrefer = {
    iOS: function iOS(callback) {
      invokeClient('app', 'readingPrefer', callback);
    },
    android: function android(callback) {
      invokeClient('app', 'readingPrefer', callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method getRecommendBookListLastShare
   * @description 获取今天书单的最后一次分享记录
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.bookListId 书单ID
   * @param {Number} callback.data.timestamp 分享时间戳(单位为秒)
   * @param {Number} callback.data.channel 分享渠道(1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var getRecommendBookListLastShare = {
    iOS: function iOS(callback) {
      invokeClient('app', 'recommendBookListLastShare', callback);
    },
    android: function android(callback) {
      invokeClient('app', 'recommendBookListLastShare', callback);
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method isLatestVersion
   * @description 判断App是否是最新版
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.latestVersion 是否最新版本(false: 不是最新版; true: 是最新版)
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var isLatestVersion = {
    iOS: function iOS(callback) {
      invokeClient('app', 'isLatestVersion', callback);
    },
    android: function android(callback) {
      invokeClient('app', 'isLatestVersion', callback);
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method openAuthor
   * @description 打开作者页
   * @param {Number} authorId 作者ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openAuthor = {
    iOS: function iOS(authorId) {
      invokeClient('app', 'openAuthor', { authorId: authorId });
    },
    android: function android(authorId) {
      invokeClient('app', 'openAuthor', { authorId: authorId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBook
   * @description 打开某书阅读页
   * @param {Number} bookId 书籍ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBook = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'openBook', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'openBook', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookComments
   * @description 打开某书评论页
   * @param {Number} bookId 书籍ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookComments = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'openBookComments', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'openBookComments', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookLastPage
   * @description 打开某书末页
   * @param {Number} bookId 书籍ID
   * @param {String} bookName 书籍名称
   * @param {String} bookStatus 完本状态(完本/未完待续)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookLastPage = {
    iOS: function iOS(bookId, bookName, bookStatus) {
      invokeClient('app', 'openBookLastPage', { bookId: bookId, bookName: bookName, bookStatus: bookStatus });
    },
    android: function android(bookId, bookName, bookStatus) {
      invokeClient('app', 'openBookLastPage', { bookId: bookId, bookName: bookName, bookStatus: bookStatus });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookList
   * @description 通用书列表页(需要客户端的服务端开发支持)
   * @param {String} url 列表URL
   * @param {groupName} groupName 列表名称
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * openBookList('/Atom.axd/Api/Recommend/GetRecommendDetail?groupId=129&channelId=9', '同类作品');
   */
  var openBookList = {
    iOS: function iOS(url, groupName) {
      invokeClient('app', 'openBookList', { url: url, groupName: groupName });
    },
    android: function android(url, groupName) {
      invokeClient('app', 'openBookList', { url: url, groupName: groupName });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookMonthTicket
   * @description 打开某书月票页
   * @param {Number} bookId 书籍ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookMonthTicket = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'openBookMonthTicket', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'openBookMonthTicket', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookRecommendTicket
   * @description 打开某书推荐票页
   * @param {Number} bookId 书籍ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookRecommendTicket = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'openBookRecommendTicket', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'openBookRecommendTicket', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookReward
   * @description 打开某书打赏页
   * @param {Number} bookId 书籍ID
   *
   * @support android 6.4.0
   */
  var openBookReward = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'openBookReward', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'openBookReward', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookStore
   * @description 打开书城
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookStore = {
    iOS: function iOS() {
      invokeClient('app', 'openBookStore');
    },
    android: function android() {
      invokeClient('app', 'openBookStore');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBookshelf
   * @description 打开书架
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBookshelf = {
    iOS: function iOS() {
      invokeClient('app', 'openBookshelf');
    },
    android: function android() {
      invokeClient('app', 'openBookshelf');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBoutiques
   * @description 打开精品专区
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openBoutiques = {
    iOS: function iOS() {
      invokeClient('app', 'openBoutiques');
    },
    android: function android() {
      invokeClient('app', 'openBoutiques');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openBugFeedback
   * @description 打开问题反馈页
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var openBugFeedback = {
    iOS: function iOS() {
      invokeClient('app', 'openBugFeedback');
    },
    android: function android() {
      invokeClient('app', 'openBugFeedback');
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method openCategory
   * @description 打开某分类页
   * @param {Number} id
   * @param {String} name
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openCategory = {
    iOS: function iOS(id, name) {
      invokeClient('app', 'openCategory', { id: id, name: name });
    },
    android: function android(id, name) {
      invokeClient('app', 'openCategory', { id: id, name: name });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openCategorys
   * @description 打开分类列表页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openCategorys = {
    iOS: function iOS() {
      invokeClient('app', 'openCategorys');
    },
    android: function android() {
      invokeClient('app', 'openCategorys');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openChapter
   * @description 打开某书某章节
   * @param {Number} bookId 书籍ID
   * @param {Number} chapterId 章节ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openChapter = {
    iOS: function iOS(bookId, chapterId) {
      invokeClient('app', 'openChapter', { bookId: bookId, chapterId: chapterId });
    },
    android: function android(bookId, chapterId) {
      invokeClient('app', 'openChapter', { bookId: bookId, chapterId: chapterId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openDailyReading
   * @description 每日导读页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openDailyReading = {
    iOS: function iOS() {
      invokeClient('app', 'openDailyReading');
    },
    android: function android() {
      invokeClient('app', 'openDailyReading');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openDiscussArea
   * @description 打开即时讨论区
   * @param {Object} params 参数对象
   * @param {String} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {Number} params.hongbaoId 定位到的红包ID，可不传
   * @param {Number} params.openHongbaoId 要打开的红包ID，可不传
   *
   * @support android 6.4.0
   */
  var openDiscussArea = {
    iOS: function iOS(params) {
      invokeClient('app', 'openDiscussArea', params);
    },
    android: function android(params) {
      invokeClient('app', 'openDiscussArea', params);
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method openFeedback
   * @description 打开意见反馈页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openFeedback = {
    iOS: function iOS() {
      invokeClient('app', 'openFeedback');
    },
    android: function android() {
      invokeClient('app', 'openFeedback');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openFilter
   * @description 筛选页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openFilter = {
    iOS: function iOS() {
      invokeClient('app', 'openFilter');
    },
    android: function android() {
      invokeClient('app', 'openFilter');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openFind
   * @description 打开发现
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openFind = {
    iOS: function iOS() {
      invokeClient('app', 'openFind');
    },
    android: function android() {
      invokeClient('app', 'openFind');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openFree
   * @description 打开今日免费
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openFree = {
    iOS: function iOS() {
      invokeClient('app', 'openFree');
    },
    android: function android() {
      invokeClient('app', 'openFree');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openLogin
   * @description 打开登录
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openLogin = {
    iOS: function iOS() {
      invokeClient('app', 'openLogin');
    },
    android: function android() {
      invokeClient('app', 'openLogin');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  var isDefined = function isDefined(d) {
    return typeof d !== 'undefined';
  };

  /**
   * @method openRank
   * @description 打开排行页
   * @param {Number} id 排行页左侧ID，可不传
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openRank = {
    iOS: function iOS(id) {
      invokeClient('app', 'openRank', isDefined(id) ? { id: id } : null);
    },
    android: function android(id) {
      invokeClient('app', 'openRank', isDefined(id) ? { id: id } : null);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openReceiveCoin
   * @description 打开领币页
   *
   * @support android 6.4.0
   */
  var openReceiveCoin = {
    android: function android() {
      invokeClient('app', 'openReceiveCoin');
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method openRecharge
   * @description 打开充值页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openRecharge = {
    iOS: function iOS() {
      invokeClient('app', 'openRecharge');
    },
    android: function android() {
      invokeClient('app', 'openRecharge');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openRechargeHistory
   * @description 打开充值记录页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openRechargeHistory = {
    iOS: function iOS() {
      invokeClient('app', 'openRechargeHistory');
    },
    android: function android() {
      invokeClient('app', 'openRechargeHistory');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openRechargeWithChannel
   * @description 打开具体渠道充值页
   * @param {String} channel 充值渠道
   * @param {Number} totalFee 充值金额
   *
   * @support android 6.4.0
   *
   * @example
   * openRechargeWithChannel('g_4_alipay_shortcut', 100); // 支付宝
   * openRechargeWithChannel('g_3_umpaypay_mobilesms', 100); // 移动手机短信充值
   * openRechargeWithChannel('g_3_huajianpay_sms9165', 100); // 电信手机短信充值
   * openRechargeWithChannel('g_3_huajianpay_sms5556', 100); // 联通手机短信充值
   * openRechargeWithChannel('g_7_payPal_USDCOST', 100); // Paypal充值
   * openRechargeWithChannel('g_8_shenzhouYD_USDCOST', 100); // 移动手机充值卡充值
   * openRechargeWithChannel('g_8_shenzhouDX_USDCOST', 100); // 电信手机充值卡充值
   * openRechargeWithChannel('g_2_WeChatPay_mobilesms', 100); // 微信充值
   * openRechargeWithChannel('g_2_TenPay_mobilesms', 100); // 财付通
   * openRechargeWithChannel('g_2_WalletPay_mobilesms', 100); // QQ钱包
   */
  var openRechargeWithChannel = {
    android: function android(channel, totalFee) {
      invokeClient('app', 'openRechargeWithChannel', { channel: channel, totalFee: totalFee });
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method openRecommendBookList
   * @description 打开某书单页
   * @param {Number} id 书单ID
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var openRecommendBookList = {
    iOS: function iOS(id) {
      invokeClient('app', 'openRecommendBookList', { id: id });
    },
    android: function android(id) {
      invokeClient('app', 'openRecommendBookList', { id: id });
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method openRecommendBookListComments
   * @description 打开某书单评论页
   * @param {Number} id 书单ID
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var openRecommendBookListComments = {
    iOS: function iOS(id) {
      invokeClient('app', 'openRecommendBookListComments', { id: id });
    },
    android: function android(id) {
      invokeClient('app', 'openRecommendBookListComments', { id: id });
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method openRecommendBookListSquare
   * @description 打开书单广场
   * @param {Number} tab 定位标签(0:最新; 1热门; 2我的书单)
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var openRecommendBookListSquare = {
    iOS: function iOS(tab) {
      invokeClient('app', 'openRecommendBookListSquare', { tab: tab });
    },
    android: function android(tab) {
      invokeClient('app', 'openRecommendBookListSquare', { tab: tab });
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method openSendRedPacket
   * @description 打开发红包页
   * @param {Number} bookId 书籍ID
   * @param {String} bookName 书籍名称
   *
   * @support android 6.4.0
   */
  var openSendRedPacket = {
    android: function android(bookId, bookName) {
      invokeClient('app', 'openSendRedPacket', { bookId: bookId, bookName: bookName });
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method openSetting
   * @description 打开设置页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openSetting = {
    iOS: function iOS() {
      invokeClient('app', 'openSetting');
    },
    android: function android() {
      invokeClient('app', 'openSetting');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openSignIn
   * @description 打开签到页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openSignIn = {
    iOS: function iOS() {
      invokeClient('app', 'openSignIn');
    },
    android: function android() {
      invokeClient('app', 'openSignIn');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openSubject
   * @description 类似精选页的通用界面(需要客户端的服务端开发支持)
   * @param {String} url 列表URL
   * @param {String} groupName 列表名称
   */
  var openSubject = {
    iOS: function iOS(url, groupName) {
      invokeClient('app', 'openSubject', { url: url, groupName: groupName });
    },
    android: function android(url, groupName) {
      invokeClient('app', 'openSubject', { url: url, groupName: groupName });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openSweep
   * @description 打开扫一扫
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openSweep = {
    iOS: function iOS() {
      invokeClient('app', 'openSweep');
    },
    android: function android() {
      invokeClient('app', 'openSweep');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openTask
   * @description 打开任务页
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openTask = {
    iOS: function iOS() {
      invokeClient('app', 'openTask');
    },
    android: function android() {
      invokeClient('app', 'openTask');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method openUserCenter
   * @description 打开个人中心
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var openUserCenter = {
    iOS: function iOS() {
      invokeClient('app', 'openUserCenter');
    },
    android: function android() {
      invokeClient('app', 'openUserCenter');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method report
   * @description 数据上报(仅用于App内嵌页面，活动页勿用)
   * @param {String} eventId 事件ID
   * @param {Number} eventType 事件类型(0:曝光; 1:点击)
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var report = {
    iOS: function iOS(eventId, eventType) {
      invokeClient('app', 'report', { eventId: eventId, eventType: eventType });
    },
    android: function android(eventId, eventType) {
      invokeClient('app', 'report', { eventId: eventId, eventType: eventType });
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method setNavigationRightButton
   * @description 自定义右上角按钮
   * @param {String} text 显示文本
   * @param {Function} callback 点击回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * setNavigationRightButton('分享', function () {
   *   share({
   *     title: '斗罗大陆',
   *     desc: '当唐门暗器来到斗罗大陆，当唐三武魂觉醒，他能否在这片武魂的世界再铸唐门的辉煌？他能否成为这个世界的主宰：神?',
   *     imageUrl: 'http://qidian.qpic.cn/qdbimg/349573/1115277/180',
   *     url: 'http://www.qidian.com/Book/1115277.aspx',
   *     channels: '1,2,3,5', // 依次显示分享渠道为：朋友圈，微信好友，QQ好友，微博
   *   });
   * });
   */
  var setNavigationRightButton = {
    iOS: function iOS(text, callback) {
      invokeClient('app', 'setNavigationRightButton', { text: text }, callback);
    },
    android: function android(text, callback) {
      invokeClient('app', 'setNavigationRightButton', { text: text }, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method share
   * @description 分享其他
   * @param {Object} params 分享配置
   * @param {String} params.title 标题(最大45字节)
   * @param {String} params.desc 描述(最大60字节)
   * @param {String} params.imageUrl 图片地址(图片最小需要200 * 200，否则分享到Qzone时会被Qzone过滤掉)
   * @param {String} params.url 链接(长度不能超过120字节，超过的请转短链)
   * @param {String} params.channels 分享渠道(用","分割, 1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   * @param {Boolean} params.hideUninstallChannel 是否隐藏未安装渠道
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   *
   * @example
   * share({
   *   title: '斗罗大陆',
   *   desc: '当唐门暗器来到斗罗大陆，当唐三武魂觉醒，他能否在这片武魂的世界再铸唐门的辉煌？他能否成为这个世界的主宰：神?',
   *   imageUrl: 'http://qidian.qpic.cn/qdbimg/349573/1115277/180',
   *   url: 'http://www.qidian.com/Book/1115277.aspx',
   *   channels: '1,2,3,5', // 依次显示分享渠道为：朋友圈，QQ空间，微信好友
   * });
   */
  var share = {
    iOS: function iOS(params) {
      invokeClient('app', 'share', params);
    },
    android: function android(params) {
      invokeClient('app', 'share', params);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method shareBook
   * @description 分享书
   * @param {Object} params 分享配置
   * @param {Number} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {String} params.bookCover 封面图片
   * @param {String} params.author 书籍作者
   * @param {String} params.channels 分享渠道(用","分割, 1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   * @param {Boolean} params.hideUninstallChannel 是否隐藏未安装渠道
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var shareBook = {
    iOS: function iOS(params) {
      invokeClient('app', 'shareBook', params);
    },
    android: function android(params) {
      invokeClient('app', 'shareBook', params);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method shareNote
   * @description 分享笔记
   * @param {Object} params 分享配置
   * @param {Number} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {String} params.bookCover 封面图片
   * @param {String} params.content 内容
   * @param {String} params.channels 分享渠道(用","分割, 1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   * @param {Boolean} params.hideUninstallChannel 是否隐藏未安装渠道
   *
   * @support android 6.4.0
   */
  var shareNote = {
    android: function android(params) {
      invokeClient('app', 'shareNote', params);
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method shareReward
   * @description 分享打赏
   * @param {Object} params 分享配置
   * @param {Number} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {String} params.bookCover 封面图片
   * @param {Number} params.count 打赏数
   * @param {String} params.channels 分享渠道(用","分割, 1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   * @param {Boolean} params.hideUninstallChannel 是否隐藏未安装渠道
   *
   * @support android 6.4.0
   */
  var shareReward = {
    android: function android(params) {
      invokeClient('app', 'shareReward', params);
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method shareVote
   * @description 分享投票
   * @param {Object} params 分享配置
   * @param {Number} params.bookId 书籍ID
   * @param {String} params.bookName 书籍名称
   * @param {String} params.bookCover 封面图片
   * @param {String} params.tickType 投票类型
   * @param {Number} params.tickCount 投票数
   * @param {String} params.channels 分享渠道(用","分割, 1: 微信朋友圈, 2: 微信好友, 3: QQ好友, 5: 微博)
   * @param {Boolean} params.hideUninstallChannel 是否隐藏未安装渠道
   *
   * @support android 6.4.0
   */
  var shareVote = {
    android: function android(params) {
      invokeClient('app', 'shareVote', params);
    },

    support: {
      android: '6.4.0'
    }
  };

  /**
   * @method showBook
   * @description 打开某书封面页
   * @param {Number} bookId 书籍ID
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var showBook = {
    iOS: function iOS(bookId) {
      invokeClient('app', 'showBook', { bookId: bookId });
    },
    android: function android(bookId) {
      invokeClient('app', 'showBook', { bookId: bookId });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method signIn
   * @description 签到
   * @param {Function} callback 回调函数
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Boolean} callback.data.signIn 签到结果(true: 签到成功; false: 签到失败)
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var signIn = {
    iOS: function iOS(callback) {
      invokeClient('app', 'signIn', callback);
    },
    android: function android(callback) {
      invokeClient('app', 'signIn', callback);
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  /**
   * @method unsetNavigationRightButton
   * @description 移除右上角自定义按钮
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var unsetNavigationRightButton = {
    iOS: function iOS() {
      invokeClient('app', 'unsetNavigationRightButton');
    },
    android: function android() {
      invokeClient('app', 'unsetNavigationRightButton');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method upgrade
   * @description App检查更新
   *
   * @support iOS 3.8.0
   * @support android 6.4.0
   */
  var upgrade = {
    iOS: function iOS() {
      invokeClient('app', 'upgrade');
    },
    android: function android() {
      invokeClient('app', 'upgrade');
    },

    support: {
      iOS: '3.8.0',
      android: '6.4.0'
    }
  };

  var app = {
    addToBookshelf: addToBookshelf,

    buyChapter: buyChapter,

    buyVolume: buyVolume,

    getLatestBook: getLatestBook,

    getReadingDuration: getReadingDuration,

    getReadingPrefer: getReadingPrefer,

    getRecommendBookListLastShare: getRecommendBookListLastShare,

    isLatestVersion: isLatestVersion,

    openAuthor: openAuthor,

    openBook: openBook,

    openBookComments: openBookComments,

    openBookLastPage: openBookLastPage,

    openBookList: openBookList,

    openBookMonthTicket: openBookMonthTicket,

    openBookRecommendTicket: openBookRecommendTicket,

    openBookReward: openBookReward,

    openBookshelf: openBookshelf,

    openBookStore: openBookStore,

    openBoutiques: openBoutiques,

    openBugFeedback: openBugFeedback,

    openCategory: openCategory,

    openCategorys: openCategorys,

    openChapter: openChapter,

    openDailyReading: openDailyReading,

    openDiscussArea: openDiscussArea,

    openFeedback: openFeedback,

    openFilter: openFilter,

    openFind: openFind,

    openFree: openFree,

    openLogin: openLogin,

    openRank: openRank,

    openReceiveCoin: openReceiveCoin,

    openRecharge: openRecharge,

    openRechargeHistory: openRechargeHistory,

    openRechargeWithChannel: openRechargeWithChannel,

    openRecommendBookList: openRecommendBookList,

    openRecommendBookListComments: openRecommendBookListComments,

    openRecommendBookListSquare: openRecommendBookListSquare,

    openSendRedPacket: openSendRedPacket,

    openSetting: openSetting,

    openSignIn: openSignIn,

    openSubject: openSubject,

    openSweep: openSweep,

    openTask: openTask,

    openUserCenter: openUserCenter,

    report: report,

    setNavigationRightButton: setNavigationRightButton,

    share: share,

    shareBook: shareBook,

    shareNote: shareNote,

    shareReward: shareReward,

    shareVote: shareVote,

    showBook: showBook,

    signIn: signIn,

    unsetNavigationRightButton: unsetNavigationRightButton,

    upgrade: upgrade
  };

  /**
   * @method getHeader
   * @description 获取当前设备信息
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {String} callback.data.header 设备信息
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getHeader = {
    iOS: function iOS(callback) {
      invokeClient('device', 'header', callback);
    },
    android: function android(callback) {
      invokeClient('device', 'header', callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method getSystemVersion
   * @description 获取系统版本号
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {String} callback.data.systemVersion 系统版本号
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getSystemVersion = {
    iOS: function iOS(callback) {
      invokeClient('device', 'systemVersion', callback);
    },
    android: function android(callback) {
      invokeClient('device', 'systemVersion', callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method getVersionCode
   * @description 获取客户端当前服务端版本号
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {String} callback.data.versionCode 客户端当前数字版本号
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var getVersionCode = {
    iOS: function iOS(callback) {
      invokeClient('device', 'versionCode', callback);
    },
    android: function android(callback) {
      invokeClient('device', 'versionCode', callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  var device = {
    getHeader: getHeader,

    getSystemVersion: getSystemVersion,

    getVersionCode: getVersionCode
  };

  /**
   * @method alert
   * @description 提示
   * @param {Object} params 设置
   * @param {String} params.title 标题
   * @param {String} params.text 提示信息
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var alert = {
    iOS: function iOS(params, callback) {
      invokeClient('ui', 'alert', params, callback);
    },
    android: function android(params, callback) {
      invokeClient('ui', 'alert', params, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method confirm
   * @description 确认框
   * @param {Object} params 设置
   * @param {String} params.title 标题
   * @param {String} params.text 确认信息
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.action 动作类型(0: 取消; 1: 确定)
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var confirm = {
    iOS: function iOS(params, callback) {
      invokeClient('ui', 'confirm', params, callback);
    },
    android: function android(params, callback) {
      invokeClient('ui', 'confirm', params, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * 关闭当前页面
   * @method exit
   * @description 关闭当前页面
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var exit = {
    iOS: function iOS() {
      invokeClient('ui', 'exit');
    },
    android: function android() {
      invokeClient('ui', 'exit');
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method popup
   * @description 对话框
   * @param {Object} params 设置
   * @param {String} params.title 标题
   * @param {String} params.text 确认信息
   * @param {Array<String>} params.buttons 按钮数组(例如：['取消', '购买'])
   * @param {Function} callback 回调方法
   * @param {Number} callback.result 回调状态(0表示正常)
   * @param {Object} callback.data 回调数据
   * @param {Number} callback.data.btnIndex 点击点击按钮索引
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var popup = {
    iOS: function iOS(params, callback) {
      invokeClient('ui', 'popup', params, callback);
    },
    android: function android(params, callback) {
      invokeClient('ui', 'popup', params, callback);
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method toast
   * @description Toast
   * @param {String} text 提示信息
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var toast = {
    iOS: function iOS(text) {
      invokeClient('ui', 'toast', { text: text });
    },
    android: function android(text) {
      invokeClient('ui', 'toast', { text: text });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  /**
   * @method toggleRefresh
   * @description 显示／隐藏右上角的刷新按钮
   * @param {Boolean} isShow 显示／隐藏, true为显示, false为隐藏
   *
   * @support iOS 3.7.3
   * @support android 6.4.0
   */
  var toggleRefresh = {
    iOS: function iOS(isShow) {
      invokeClient('ui', 'toggleRefresh', { isShow: isShow });
    },
    android: function android(isShow) {
      invokeClient('ui', 'toggleRefresh', { isShow: isShow });
    },

    support: {
      iOS: '3.7.3',
      android: '6.4.0'
    }
  };

  var ui = {
    alert: alert,

    confirm: confirm,

    exit: exit,

    popup: popup,

    toast: toast,

    toggleRefresh: toggleRefresh
  };

  sdkRoot.version = '1.0.0';

  function mountModules(modules) {
    Object.keys(modules).forEach(function (moduleName) {
      var methods = modules[moduleName];

      Object.keys(methods).forEach(function (methodName) {
        mount(moduleName, methodName, methods[methodName]);
      });
    });
  }

  mountModules({
    app: app,
    device: device,
    ui: ui
  });

  // 供客户端调用回调方法，例如: execCallback(callbackId, data)
  window.execCallback = execCallback;

  // 供客户端触发事件，例如: dispatchEvent(eventName, data)
  window.dispatchEvent = dispatchEvent;

  return sdkRoot;

}));