/**
 * SVG离线
 *
 * @author Zhang Xinxu & Zhang Chengrong
 *
 * @examples
 * requireSVG('path/to/a.svg');
 * requireSVG('path/to/a.svg?v=1');
 * requireSVG('path/to/a.0.1.svg');
 */

(function (global, factory) {
    if (typeof exports === 'object' && typeof module !== 'undefined') {
        module.exports = factory();
    } else if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        global.requireSVG = factory();
    }
}(this, function () {
    'use strict';

    function each(arr, fn) {
        if (!(arr instanceof Array) || typeof fn !== 'function') {
            throw new Error('"arr"必须为数组, "fn"必须为方法');
        }

        if ([].forEach) {
            arr.forEach(fn);
        } else {
            for (var i = 0, n = arr.length; i < n; i += 1) {
                fn.call(null, arr[i], i);
            }
        }
    }

    /**
     * 将请求地址分隔为文件名和版本号
     * @param  {String} url 请求地址，支持 "a.svg?v=1" 和 "a.0.1.svg" 两种格式
     * @return {Object<base[String], version[String]>} 文件名和版本号
     */
    function split(url) {
        if (typeof url !== 'string') {
            throw new Error('"url"必须为字符串');
        }

        var base;
        var version;

        if (url.indexOf('?') > -1) {
            base = url.split('?')[0];
            version = url.split('?')[1];
        } else {
            var ret = url.match(/([\d\.]+)\.svg/);
            if (ret) {
                version = ret[1];
                base = url.replace(version, '');
            } else {
                base = url;
            }
        }

        return {
            base: base,
            version: version,
        };
    }

    /**
     * 加载SVG
     * @param {String} url 加载地址
     * @param {Function} callback 回调函数
     */
    function loadSVG(url, callback) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.onload = function () {
            if (/svg/.test(xhr.responseText)) {
                callback(xhr.responseText);
            }
        };
        xhr.send();
    }

    /**
     * 加载SVG并离线存储
     * @param {Array|String} urls SVG地址数组
     * @param {Function} callback 回调函数(每一个SVG加载成功都会调用)
     */
    return function (urls, callback) {
        if (!urls) {
            return;
        }

        if (typeof urls === 'string') {
            urls = [urls];
        }

        function handler(svgText) {
            if (typeof callback === 'function') {
                callback(svgText);
            } else {
                var div = document.createElement('div');
                div.style.display = 'none';
                div.innerHTML = svgText;
                document.body.insertBefore(div, document.body.firstChild);
            }
        }

        each(urls, function (url) {
            var splitedUrlObj = split(url);
            var base = splitedUrlObj.base;
            var version = splitedUrlObj.version;

            var storeText = localStorage.getItem(base);
            var storeVersion = localStorage.getItem(base + '-v');

            if (storeText && storeVersion === version) {
                handler(storeText);
            } else {
                loadSVG(url, function (svgText) {
                    handler(svgText);

                    localStorage.setItem(base, svgText);

                    if (version) {
                        localStorage.setItem(base + '-v', version);
                    }
                });
            }
        });
    };
}));
